VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsNTLog"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'API functions used for error logging.
Private Declare Function RegisterEventSource Lib "advapi32.dll" Alias _
                         "RegisterEventSourceA" (ByVal lpUNCServerName As String, _
                                                 ByVal lpSourceName As String) As Long
Private Declare Function DeregisterEventSource Lib "advapi32.dll" _
                         (ByVal hEventLog As Long) As Long
Private Declare Function ReportEvent Lib "advapi32.dll" Alias "ReportEventA" _
                         (ByVal hEventLog As Long, ByVal wType As Integer, _
                         ByVal wCategory As Integer, ByVal dwEventID As Long, _
                         ByVal lpUserSid As Any, ByVal wNumStrings As Integer, _
                         ByVal dwDataSize As Long, plpStrings As Long, _
                         lpRawData As Any) As Boolean
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
                    (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
Private Declare Function GlobalAlloc Lib "kernel32" (ByVal wFlags As Long, _
                                                     ByVal dwBytes As Long) As Long
Private Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long

Private Const GMEM_ZEROINIT As Long = &H40
'========================================================================================
' LogNTEvent
'========================================================================================
'
' Purpose:  Writes error details to an NT Application Log.
' Inputs :  lngEventType - the type of the event to log.
'           lngMessageID - the message to be logged.
'           strInsertionStrings - a string array containing the text to be inserted into
'                                 the message.
'           bytData - a byte array containing bytes to be logged.
' Assumes:  None.
' Returns:  Boolean - indicating the success of the call.
' Effects:  None.
'
'========================================================================================
Public Function LogNTEvent(ByVal lngEventType As Long, _
                           ByVal lngMessageID As Long, _
                           ByRef strInsertionStrings() As String, _
                           ByRef bytData() As Byte _
                           ) As Boolean

    Const strLOCAL_MACHINE As String = ""
    Const lngNULL_CHAR_LENGTH As Long = 1
    Const intNO_CATEGORY As Integer = 0
    Const lngNO_USER_ID As Long = 0&
    Const lngARRAY_POINTER As Long = 0
    Const strEVENT_LOG As String = "TestNTEventLogMessage"

    Dim lngEventLog As Long
    Dim intInsertionStringCount As Integer
    Dim lngArrayCount As Long
    Dim lngTotalStringSize As Long
    Dim lngBaseMemoryAddress As Long
    Dim lngNextStringAddress As Long
    Dim lngNextStringLength As Long
    Dim lngStringPointers() As Long
    Dim lngDataSize As Long
    Dim blnResult As Long

    On Error GoTo LOGNTEVENT_EXIT
    
    'Register the event source.
    lngEventLog = RegisterEventSource(strLOCAL_MACHINE, strEVENT_LOG)
    
    'Determine the number of insertion strings (array is 0 based).
    intInsertionStringCount = UBound(strInsertionStrings) + 1

    'Determine the length of all the insertion strings to be logged including space
    'for the terminating Null characters.
    For lngArrayCount = 0 To intInsertionStringCount - 1
       lngTotalStringSize = lngTotalStringSize + _
                                            Len(strInsertionStrings(lngArrayCount)) + 1
    Next lngArrayCount

    'Get a handle to an area of memory big enough to store the insertion strings.
    lngBaseMemoryAddress = GlobalAlloc(GMEM_ZEROINIT, lngTotalStringSize)

    'Set the memory location for the next string to be copied to.
    lngNextStringAddress = lngBaseMemoryAddress

    'Dimension the pointers array to the number of insertion strings.
    ReDim lngStringPointers(intInsertionStringCount - 1)

    'We will use CopyMemory to copy the strings from their current memory location to
    'the memory we allocated above. We must do this because VB stores strings in
    'UNICODE (2 bytes per character).
    
    'Take the insertion string "My Test" as an example.

    'If we looked at the string via its current address, we would see 14 bytes, with the
    'first two bytes containing "M" and " " respectivley.  The ReportEvent function would
    'take the second byte as a null terminator to the string, and only log the "M".

    'When using CopyMemory the Visual Basic run-time library converts strings from
    'UNICODE to ANSI (1 byte per character), so removing the null character problem.

    'The last point is that the ReportEvent function requires an array of pointers to
    'strings.  So while the strings are being copied we will take a copy of the target
    'memory address and store it in an array.
    
    For lngArrayCount = 0 To intInsertionStringCount - 1
        'Create a pointer to the string.
        lngStringPointers(lngArrayCount) = lngNextStringAddress
        
        'Get the length of the current insertion string.
        lngNextStringLength = Len(strInsertionStrings(lngArrayCount))

        'Copy the insertion string, and then copy a null onto the end.
        Call CopyMemory(ByVal lngNextStringAddress, _
                        ByVal strInsertionStrings(lngArrayCount), lngNextStringLength)
        
        Call CopyMemory(ByVal lngNextStringAddress + lngNextStringLength, _
                        vbNullString, lngNULL_CHAR_LENGTH)
        
        'Increment the next string address (including the null character).
        lngNextStringAddress = lngNextStringAddress + lngNextStringLength + _
                                                                    lngNULL_CHAR_LENGTH
    Next lngArrayCount

    'DEBUG CODE.
    'Use this code to see the string that has been created.
    'Dim strCopiedText As String
    'strCopiedText = Space(lngTotalStringSize)
    'Call CopyMemory(ByVal strCopiedText, ByVal lngBaseMemoryAddress, lngTotalStringSize)
    'Debug.Print strCopiedText

    'Determine the number of data bytes to be logged.
    lngDataSize = UBound(bytData) + 1

    'Log the event.
    'Instead of passing the array of pointers we need to pass a pointer to the array.
    'Passing the first element of the array achieves this.
    blnResult = ReportEvent(lngEventLog, lngEventType, intNO_CATEGORY, lngMessageID, _
                            lngNO_USER_ID, intInsertionStringCount, lngDataSize, _
                            lngStringPointers(lngARRAY_POINTER), _
                            bytData(lngARRAY_POINTER))
    
LOGNTEVENT_EXIT:
   'Free the previously allocated memory.
   Call GlobalFree(lngBaseMemoryAddress)
   
   'Unregister the event.
   Call DeregisterEventSource(lngEventLog)

End Function
'========================================================================================
' LogNTEventSimple
'========================================================================================
'
' Purpose:  Writes error details to an NT Application Log using the App object.
' Inputs :  strTextToLog - the text to log.
'           lngLogType - OPTIONAL - specifies the type of log entry, if ommitted
'                        Information is used as default (this matches VBs behaviour).
' Assumes:  None.
' Returns:  None.
' Effects:  None.
'
'========================================================================================
Public Sub LogNTEventSimple(ByVal strTextToLog As String, _
                            Optional ByVal lngLogType As Long = vbLogEventTypeInformation)

    App.LogEvent strTextToLog

End Sub


